# utils.py

import random
import re
import datetime
from urllib.parse import quote

def generate_unique_tracking_code():
    """تولید کد پیگیری یکتا"""
    try:
        while True:
            code = ''.join(random.choices('0123456789', k=8))
            conn = connect_to_database()
            if conn:
                cursor = conn.cursor()
                cursor.execute("SELECT COUNT(*) FROM users WHERE tracking_code = %s", (code,))
                if cursor.fetchone()[0] == 0:
                    conn.close()
                    return code
                conn.close()
    except Exception as e:
        logging.error(f"خطا در تولید کد پیگیری: {e}")
        return None

def validate_national_id(national_id):
    """اعتبارسنجی کد ملی"""
    if not re.match(r'^\d{10}$', national_id):
        return False

    check = int(national_id[-1])
    temp = sum(int(national_id[i]) * (10 - i) for i in range(9)) % 11

    return (temp < 2 and check == temp) or (temp >= 2 and check + temp == 11)

def validate_phone(phone):
    """اعتبارسنجی شماره موبایل"""
    return bool(re.match(r'^09\d{9}$', phone))

def validate_date(date_str):
    """اعتبارسنجی تاریخ تولد"""
    try:
        day, month, year = map(int, date_str.split('/'))
        if not (1300 <= year <= 1402):
            return False
        datetime.date(year, month, day)
        return True
    except:
        return False

def validate_link(link):
    """اعتبارسنجی لینک"""
    return bool(re.match(r'^https?://(?:[\w-]+\.)+[\w-]+(?:/[\w\-\.\?\%\&\=]*)?$', link))

def check_existing_data(field_name, value):
    """بررسی وجود داده تکراری در دیتابیس"""
    try:
        conn = connect_to_database()
        if conn:
            cursor = conn.cursor()
            cursor.execute(f"SELECT COUNT(*) FROM users WHERE {field_name} = %s", (value,))
            count = cursor.fetchone()[0]
            conn.close()
            return count > 0
        return False
    except Exception as e:
        logging.error(f"خطا در بررسی داده تکراری: {e}")
        return False

def get_markup(back=True, home=False):
    """ایجاد دکمه‌های مناسب برای هر مرحله"""
    markup = types.ReplyKeyboardMarkup(resize_keyboard=True, row_width=2)
    buttons = []

    if back:
        buttons.append(types.KeyboardButton("↩️ برگشت"))
    if home:
        buttons.append(types.KeyboardButton("🏠 منوی اصلی"))

    markup.add(*buttons)
    return markup
